@@ -30,8 +30,6 @@ def register():
         Period,
         Move,
         Reconciliation,
-        Configuration,
-        ConfigurationTaxRounding,
         module='account_invoice', type_='model')
     Pool.register(
         TestPaymentTerm,
